package com.chelvc.framework.security.context;

import java.util.Set;
import java.util.stream.Stream;
import javax.crypto.Cipher;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;

/**
 * 应用安全相关操作上下文工具类
 *
 * @author Woody
 * @date 2024/5/24
 */
public final class SecurityContextHolder {
    /**
     * 默认权限分组
     */
    public static final String DEFAULT_PERMISSION_GROUP = "DEFAULT";

    /**
     * 数据加解密上下文
     */
    private static SecurityCryptoContext CRYPTO_CONTEXT;

    private SecurityContextHolder() {
    }

    /**
     * 获取数据加解密上下文
     *
     * @return 数据加解密上下文实例
     */
    public static SecurityCryptoContext getCryptoContext() {
        if (CRYPTO_CONTEXT == null) {
            synchronized (SecurityCryptoContext.class) {
                if (CRYPTO_CONTEXT == null) {
                    CRYPTO_CONTEXT = ApplicationContextHolder.getBean(SecurityCryptoContext.class);
                }
            }
        }
        return CRYPTO_CONTEXT;
    }

    /**
     * 获取加密处理器
     *
     * @return 加密处理器
     */
    public static Cipher getEncryptor() {
        return getCryptoContext().getEncryptor();
    }

    /**
     * 获取解密处理器
     *
     * @return 解密处理器
     */
    public static Cipher getDecrypter() {
        return getCryptoContext().getDecrypter();
    }

    /**
     * 数字签名
     *
     * @param session 会话信息
     * @param payload 请求数据
     * @return 签名信息
     */
    public static String sign(@NonNull Session session, String payload) {
        String secret = getCryptoContext().getSecret();
        String plaintext = secret + session.getPlatform() + session.getTerminal() + session.getVersion() +
                session.getTimestamp() + payload;
        return CodecUtils.md5(plaintext);
    }

    /**
     * 判断当前用户是否拥有任意权限
     *
     * @param permissions 权限标识数组
     * @return true/false
     */
    public static boolean hashAnyPermission(@NonNull String... permissions) {
        if (ObjectUtils.isEmpty(permissions)) {
            return true;
        }
        Session session = SessionContextHolder.getSession(false);
        Set<String> authorities = ObjectUtils.ifNull(session, Session::getAuthorities);
        if (ObjectUtils.notEmpty(authorities)) {
            Stream<String> stream = Stream.of(permissions);
            if (stream.anyMatch(authorities::contains)) {
                return true;
            }
            for (String authority : authorities) {
                Set<?> properties = ApplicationContextHolder.getProperty(authority, Set.class);
                if (ObjectUtils.notEmpty(properties) && stream.anyMatch(properties::contains)) {
                    return true;
                }
            }
        }
        return false;
    }
}
